"""
Copyright 2022 Raphaël Léas
University of Liège

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import subprocess
import os
import signal

COMMAND = 'sudo tcpdump -i {} -w {} -U icmp6 or ip6 and not port 22 and not port 5353 and not port 53'

class Capture:
    """ Capture class
    This class is responsible of capturing IN/OUT packets by creating a tcpdump instance

    Attributes
    ----------
    interface : str
        The interface name to listen

    outFilename : str
        The name of the PCAP file collecting IN/OUT packets
    """

    def __init__(self, interface, out_filename):
        self.interface = interface
        self.outFilename = out_filename

    """
    Start tcpdump capture
    """
    def start_capture(self): 
        self.process = subprocess.Popen(COMMAND.format(self.interface, self.outFilename).split(), shell=False, cwd=os.getcwd(), stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

    """
    Stop tcpdump capture
    """ 
    def stop_capture(self):
        self.process.send_signal(signal.SIGINT)
