"""
Copyright 2022 Raphaël Léas
University of Liège

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import socket
import argparse
import subprocess
import os
import signal

COMMAND = "tcpdump -i {} -w {} ip6 and dst {} and not icmp6 and not port 22"
OUTFILENAME = 'received_traffic.pcap'

"""
Parse program arguments
"""
def parseArgs():
    parser = argparse.ArgumentParser()

    # Interface to listen to
    parser.add_argument('--i', type=str, required=True, help='The name of the interface to listen')
    # Host
    parser.add_argument('--host', type=str, default='::', help='The host address of the interface')
    # List of porst
    parser.add_argument('--pList', nargs='+', default=['33434'], help='The list of ports to open')

    args = parser.parse_args()
    return args

"""
Open a socket on a specific port
"""
def create_socket(host, port):
    # Create a IPv6 socket in tcp mode
    serverSocket = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
    # Bind the socket to a public host and a port
    serverSocket.bind((host, port))

    return serverSocket

"""
Main function
"""
def main():
    sockets = []

    # Parse arguments
    args = parseArgs()
    host = args.host
    ports = args.pList
    interface = args.i

    # Start tcpdump
    capture = subprocess.Popen(COMMAND.format(interface, OUTFILENAME, host).split(), shell=False, cwd=os.getcwd(), stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

     # Set up the closing of tcpdumb on ctrl+c
    def ctrlc_handler(signum, frame):
        capture.send_signal(signal.SIGINT)
        exit()

    signal.signal(signal.SIGINT, ctrlc_handler)

    # Create the sockets
    for port in ports:
        sockets.append(create_socket(host, int(port)))

    print("[INFO] Opened sockets on ports:")
    for port in ports:
        print(port)
    
    print('[INFO] Waiting for client requests...')
    while True:
        # Listening
        for s in sockets:
            s.listen(5)

if __name__ == "__main__":
    main()

